;*****************************************************************************
;*
;*  VERSION CONTROL:    @(#)cstart.inc	1.29 01/10/17
;*
;*  DESCRIPTION:        DSP56xxx C-compiler startup code.
;*
;*  IN PACKAGE:         DSP563xx C-compiler library.
;*
;*  COPYRIGHT:          Copyright (c) 1996-2001 TASKING, Inc.
;*
;*****************************************************************************

		page	132

;	DEFINES to tune the startup code (<target_name>.asm):
;
;	NODSTACK	-> Do NOT initialize the (dynamic) user stack pointer.
;	NOESTACK	-> Do NOT initialize stack extension hardware.
;	NOCACHE 	-> Do NOT enable cache.
;	NOCOPY		-> Do NOT produce code to clear BSS sections and
;			   initialize DATA sections.
;	NOARGV		-> Do NOT produce code to initialize arguments to
;			   function main().
;	PLLVALUE=<val>	-> Set clock phase locked loop register to <val>. If
;			   not defined, the PLL is set to the default for the 
;			   selected board.
;	PLL_CLOCK=<val>	-> Set clock control register of the DPLL (56321)
;			   to <val>. PLLVALUE must be cleared to enable DPLL!
;	PLL_STATIC=<val>-> Set static control register of the DPLL (56321)
;			   to <val>. PLLVALUE must be cleared to enable DPLL!
;	LOW_DYNAMIC	-> Select 16-bit arithmetic mode.
;	NARROW_BUS	-> Select 16-bit address mode (5600x compatibility).
;	NOFSS		-> Do NOT use the file system simulation features of
;			   CrossView.
;	USP=[R6|R7]	-> Select register r6 or r7 for user stack pointer.
;
; By default, the startup code will clear the BSS sections and initialize DATA
; sections, dynamic stack pointer, stack extension and cache.

		extern	Fmain,Fexit

		extern	null_x,null_y
		extern  start,irq1,irq2
;		extern  irq3,irq4,irq5,irq6,irq7,irq8,irq9,irq10
;		extern  irq11,irq12,irq13,irq14,irq15,irq16,irq17,irq18,irq19
;		extern  irq20,irq21,irq22,irq23,irq24,irq25,irq26,irq27,irq28
;		extern  irq29,irq30,irq31,irq32,irq33,irq34,irq35,irq36,irq37
;		extern  irq38,irq39,irq40,irq41,irq42,irq43,irq44,irq45,irq46
;		extern  irq47,irq48,irq49,irq50,irq51,irq52,irq53,irq54,irq55
;		extern  irq56,irq57,irq58,irq59,irq60,irq61,irq62,irq63

; The interrupt vector table can be filled with default vectors by the linker
; for all positions for which no vector is provided by the compiler:
; a contiguous range of user vectors can be uncommented if required.


                IF      @DEF(_AS56)
                define  DSPTYPE     '@DSP()'
		org	p,'.pitext',internal:
                ELSE
		IF	!@DEF('DSPTYPE')	; define on command line for Motorola tools!
                define  DSPTYPE     '3'
		ENDIF
                org     pi:                     ; first part must reside in internal memory
                ENDIF

		global	F_START
F_START:
                IF      DSPTYPE==3              ; DSP563xx:
		IF	R_AAR0VALUE!=0		; initialize external memory
		movep	#R_AAR0VALUE,x:AAR0
		ENDIF	; R_AAR0VALUE
		IF	R_AAR1VALUE!=0
		movep	#R_AAR1VALUE,x:AAR1
		ENDIF	; R_AAR1VALUE
		IF	R_AAR2VALUE!=0
		movep	#R_AAR2VALUE,x:AAR2
		ENDIF	; R_AAR2VALUE
		IF	R_AAR3VALUE!=0
		movep	#R_AAR3VALUE,x:AAR3
		ENDIF	; R_AAR3VALUE
		IF	R_DCRVALUE!=0
		movep	#R_DCRVALUE,x:DCR
		ENDIF	; R_DCRVALUE
                ENDIF   ; DSPTYPE==3

                IF      R_OMRVALUE!=0  		; initialize memory switches
                move    #R_OMRVALUE,OMR
                ENDIF   ; R_OMRVALUE
                IF      R_SRVALUE!=0
		IF	R_SRVALUE==$C80300	; avoid overhead of code below
		bset	#19,SR			;  if only 'cache on' is required
		ELSE	; R_SRVALUE==$C80300
		movec	SR,a			; set SR to R_SRVALUE,
		and	#$300,a			;  but leave irq mask unchanged
		or	#(R_SRVALUE&~$300),a	; (required for timer cycle counting)
                move    a,SR
		ENDIF	; R_SRVALUE==$C80300
                ENDIF   ; R_SRVALUE

		jmp	ext_start


                IF      @DEF(_AS56)
		org	p,'.petext',external:	; non-critical, put external if necessary
                ELSE
                org	p:			; do not force to external memory
                ENDIF

		local	ext_start
ext_start:	
		IF	VBAVALUE!=0
		move	#VBAVALUE,VBA		   ; initialize Vector Base Address
		ENDIF	; VBAVALUE!=0

		extern	setvectorbase		   ; link in a file from runtime lib that
R__VBAVALUE	equ	VBAVALUE		   ; is used to set R_VBAVALUE and
		global	R__VBAVALUE		   ; exports it for interrupt vectors
		
                IF      DSPTYPE==3 ;-------------- if-part: startup for DSP563xx
		
                IF	@DEF(BCR)&&(R_BCRVALUE!=$FFFFFF)
		movep	#R_BCRVALUE,x:BCR          ; initialize memory wait states
		ENDIF	; @DEF(BCR)&&(R_BCRVALUE!=$FFFFFF)

		IF	PLLVALUE!=0 ;------------- if-part: PLL value for DSP563xx

                IF	@DEF(PCTL)
		movep	#PLLVALUE,x:PCTL           ; initialize clock phase locked loop
		ENDIF	; @DEF(PCTL)

		ELSE	; PLLVALUE!=0 ------------ else-part: DPLL value for DSP56321

                IF	@DEF(DCCR)&&@DEF(DSCR)
		movep	#PLL_CLOCK,x:DCCR          ; initialize DPLL clock control register
		movep	#PLL_STATIC,x:DSCR         ; initialize DPLL static control register
		ENDIF	; @DEF(DCCR)&&@DEF(DSCR)

		ENDIF	; PLLVALUE!=0

                ELSE    ; DSPTYPE==3 ------------- else-part: startup for DSP566xx

		IF	@DEF(BCR)&&(R_BCRVALUE!=$FFFF)
		movep	#R_BCRVALUE,x:BCR          ; initialize memory wait states
		ENDIF	; @DEF(BCR)&&(R_BCRVALUE!=$FFFF)

		IF	@DEF(PCTL0)&&(PLLVALUE!=0) ; initialize clock phase locked loop
		movep	#(PLLVALUE&$FFFF),x:PCTL0  ; low 16 bits to PCTL0
		movep	#(PLLVALUE>>16),x:PCTL1    ; hi  16 bits to PCTL1
		ENDIF	; @DEF(PCTL0)&&(PLLVALUE!=0)

                ENDIF   ; DSPTYPE==3 ------------- end of memory initialisation part


		IF	!@DEF('NODSTACK')	; initialize dynamic stack pointer
		extern	F_lc_bs,F_lc_es
		IF	@DEF('USP')		; (USP determined on command line)
		move	#F_lc_bs,USP
		ELSE	; @DEF('USP')		; (USP determined externally)
		extern	R_STKINIT		; opcode for stack initialisation
		dc	R_STKINIT,F_lc_bs	; this codes "move #F_lc_bs,USP" 
		ENDIF   ; @DEF('USP')
		ENDIF	; !@DEF('NODSTACK')

		IF	!@DEF('NOESTACK')	; initialize stack extension
		extern	F_lc_ub_se,F_lc_ue_se
		move	#F_lc_ub_se+2,ep        ; +2 for DSP5630x silicon bug
		move	#(F_lc_ue_se-(F_lc_ub_se+2))/2+14,sz
                IF      DSPTYPE==3
		IF	@DEFMEM()!='y'
		bset    #16,OMR                 ; set SXY (Y stack extension)
		ENDIF	; @DEFMEM()!='y'
		bset	#20,OMR			; set SEN (enable stack extension)
		ELSE    ; DSPTYPE==3
		IF	@DEFMEM()!='y'
		bset    #8,OMR                  ; set SXY (Y stack extension)
		ENDIF	; @DEFMEM()!='y'
		bset	#12,OMR			; set SEN (enable stack extension)
		ENDIF   ; DSPTYPE==3
                ENDIF	; !@DEF('NOESTACK')
	
		IF	!@DEF('NOCOPY')		; initialize static variables and bss
		extern	F_lc_cp
		clr	a	#F_lc_cp,r2
_cptable_loop:	move		p:(r2)+,b	; flags + memory type
		move		p:(r2)+,r0	; dest
		move		p:(r2)+,r4	; source
		move		p:(r2)+,y0	; count (words; for L: lwords)
		jsset	#0,b1,cptable_copy	; (flags & 1) == 1: copy block
		jsset	#1,b1,cptable_clr	; (flags & 2) == 2: clear block
		tst	b
		jne	_cptable_loop		; flags != 0: continue
		ENDIF	; !@DEF('NOCOPY')

		IF	@DEF('LOW_DYNAMIC')
		bset	#17,sr			; 16-bit arithmetic mode
		ENDIF	; @DEF('LOW_DYNAMIC')

		IF	@DEF('NARROW_BUS')
		bset	#13,sr			; 16-bit compatibility mode
		ENDIF	; NARROW_BUS

		IF	!@DEF('NOARGV')
		move	#>1,a			; argument count
		move	#argv,r0		; argument vector
		ENDIF	; !NOARGV

		jsr	Fmain			; execute user program

		jmp	Fexit			; 'call' exit() if it isn't explicitly called


		IF	!@DEF('NOCOPY')
cptable_copy:	jset	#10,b1,_cptable_cpy_l	; l ^= 4
		jclr	#8,b1,_cptable_cpy_x	; x ^= 2
		jclr	#9,b1,_cptable_cpy_p	; p ^= 1
						; y ^= 3
; memcpy(y:r0, p:r4, y0)
_cptable_cpy_y:	do	y0,_cptable_clp_y	; count-- 
		move	p:(r4)+,x1		; *source++
		move		x1,y:(r0)+	; *dest++
_cptable_clp_y: rts

; memcpy(p:r0, p:r4, y0)
_cptable_cpy_p:	do	y0,_cptable_clp_p	; count--
		move	p:(r4)+,x1		; *source++
		move	x1,p:(r0)+		; *dest++
                nop
_cptable_clp_p:	rts

; memcpy(x:r0, p:r4, y0)
_cptable_cpy_x:	do	y0,_cptable_clp_x	; count--
		move	p:(r4)+,x1		; *source++
		move		x1,x:(r0)+	; *dest++
_cptable_clp_x: rts

; memcpy(l:r0, p:r4, y0)
_cptable_cpy_l:	do	y0,_cptable_clp_l	; count--
		move	p:(r4)+,x0		; *source++
		move	p:(r4)+,x1
		move		x,l:(r0)+	; *dest++
_cptable_clp_l: rts


cptable_clr:	jset	#10,b1,_cptable_clr_l	; l ^= 4
		jclr	#8,b1,_cptable_clr_x	; x ^= 2
		jclr	#9,b1,_cptable_clr_p	; p ^= 1
						; y ^= 3
; memset(y:r0, a, y0)
_cptable_clr_y:	rep	y0			; count--
		move		a,y:(r0)+	; *dest++ = 0
		rts

; memset(p:r0, a, y0)
_cptable_clr_p:	rep	y0			; count--
		move	a,p:(r0)+		; *dest++ = 0
		rts

; memset(x:r0, a, y0)
_cptable_clr_x:	rep	y0			; count--
		move		a,x:(r0)+	; *dest++ = 0
		rts

; memset(l:r0, a, y0)
_cptable_clr_l:	rep	y0			; count--
		move		a,l:(r0)+	; *dest++ = 0
		rts

		local	cptable_copy,cptable_clr
		ENDIF	; !NOCOPY


		IF	!@DEF('NOARGV')
                IF      @DEF(_AS56)
		org	x,".xeconst",external,const:
                ELSE
                org     x:
                ENDIF
argv:		dc	prgname
prgname:	dc	0		; empty string _and_ NULL pointer
		local	argv,prgname
		ENDIF	; !NOARGV

;*****************************************************************************
